SystemOrganization addCategory: #'Metacello-GitHub'!

MCFileTreeRepository subclass: #MCGitHubRepository
	instanceVariableNames: 'projectPath projectVersion repoPath'
	classVariableNames: 'DownloadCache'
	poolDictionaries: ''
	category: 'Metacello-GitHub'!

!MCGitHubRepository class methodsFor: 'private' stamp: 'ChristopheDemarey 4/29/2013 15:23'!
cacheDirectory
	| defaultDirectory cacheDirectory |
	
	defaultDirectory := MetacelloPlatform current defaultDirectory.
	cacheDirectory := MetacelloPlatform current  directoryFromPath: 'github-cache' relativeTo: defaultDirectory.
	MetacelloPlatform current ensureDirectoryExists: cacheDirectory.
	^cacheDirectory! !

!MCGitHubRepository class methodsFor: 'private' stamp: 'ChristopheDemarey 4/29/2013 15:24'!
cacheDirectoryFor: projectPath
	| cacheDirectory projectDirectory |
	
	cacheDirectory := self cacheDirectory.
	projectDirectory := MetacelloPlatform current directoryFromPath: projectPath relativeTo: cacheDirectory.
	MetacelloPlatform current ensureDirectoryExists: projectDirectory.
	^ projectDirectory! !

!MCGitHubRepository class methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
description
    ^ 'github://'! !

!MCGitHubRepository class methodsFor: 'accessing' stamp: 'dkh 6/16/2012 08:29'!
downloadCache
    DownloadCache ifNil: [ DownloadCache := Dictionary new ].
    ^ DownloadCache! !

!MCGitHubRepository class methodsFor: 'accessing' stamp: 'dkh 7/24/2012 21:50'!
downloadCacheKey: projectPath version: versionString
    ^ projectPath , ':::' , versionString! !

!MCGitHubRepository class methodsFor: 'initialization' stamp: 'dkh 6/16/2012 08:25'!
flushDownloadCache
    "self flushDownloadCache"

    DownloadCache := nil! !

!MCGitHubRepository class methodsFor: 'utility' stamp: 'dkh 7/24/2012 21:50'!
flushProjectEntry: projectPath version: versionString
    self downloadCache removeKey: (self downloadCacheKey: projectPath version: versionString) ifAbsent: [  ]! !

!MCGitHubRepository class methodsFor: 'initialization' stamp: 'dkh 6/16/2012 08:26'!
initialize
    "self initialize"

    Smalltalk addToStartUpList: self.
    self flushDownloadCache! !

!MCGitHubRepository class methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
isAbstract
    "abstract as far as creating new repositories interactively? yes"

    ^ true! !

!MCGitHubRepository class methodsFor: 'instance creation' stamp: 'dkh 5/16/2012 14:52:41'!
location: locationUrl
    ^ self location: locationUrl version: nil! !

!MCGitHubRepository class methodsFor: 'instance creation' stamp: 'dkh 6/16/2012 09:08'!
location: locationUrl version: versionString
    ^ self parseLocation: locationUrl version: versionString! !

!MCGitHubRepository class methodsFor: 'private' stamp: 'dkh 5/16/2012 14:52:41'!
parseLocation: locationUrl version: versionString
    "self 
	parseLocation: 'github://dalehenrich/MetacelloRepository:master/monticello/repos/itory/path'
	version: nil
     "

    | projectPath projectVersion repoPath headerSize desc projectDelim repoDelim versionDelim |
    headerSize := 'github://' size.
    desc := locationUrl.
    desc := desc copyFrom: headerSize + 1 to: desc size.
    projectVersion := repoPath := nil.
    projectDelim := desc indexOf: $/.
    repoDelim := desc indexOf: $/ startingAt: projectDelim + 1.
    (versionDelim := desc indexOf: $:) == 0
        ifTrue: [ 
            repoDelim == 0
                ifTrue: [ projectPath := desc ]
                ifFalse: [ 
                    projectPath := desc copyFrom: 1 to: repoDelim - 1.
                    repoPath := desc copyFrom: repoDelim + 1 to: desc size ] ]
        ifFalse: [ 
            projectPath := desc copyFrom: 1 to: versionDelim - 1.
            repoDelim == 0
                ifTrue: [ projectVersion := desc copyFrom: versionDelim + 1 to: desc size ]
                ifFalse: [ 
                    projectVersion := desc copyFrom: versionDelim + 1 to: repoDelim - 1.
                    repoPath := desc copyFrom: repoDelim + 1 to: desc size ] ].
    versionString ~~ nil
        ifTrue: [ projectVersion := versionString ].
    ^ self new
        projectPath: projectPath;
        projectVersion: projectVersion;
        repoPath: repoPath;
        yourself
! !

!MCGitHubRepository class methodsFor: 'utility' stamp: 'ChristopheDemarey 4/29/2013 15:34'!
projectDirectoryFrom: projectPath version: versionString
	| mcPlatform githubCacheDirectory projectDirectory downloadCacheKey cachePath |
	downloadCacheKey := self downloadCacheKey: projectPath version: versionString.
	mcPlatform := MetacelloPlatform current.
	githubCacheDirectory := mcPlatform directoryFromPath: versionString relativeTo: (self cacheDirectoryFor: projectPath).
	cachePath := self downloadCache at: downloadCacheKey ifAbsent: [  ].
	(cachePath isNil or: [ (projectDirectory := mcPlatform directoryFromPath: cachePath relativeTo: githubCacheDirectory) exists not ])
		ifTrue: [ 
			| url archive directory |
			MetacelloScriptGitHubDownloadNotification new
				projectPath: projectPath;
				versionString: versionString;
				signal.	"for testing purposes"
			mcPlatform ensureDirectoryExists: githubCacheDirectory.
			url := 'https://github.com/' , projectPath , '/zipball/' , versionString.
			archive := MetacelloPlatform current downloadFile: url to: '/tmp/github.zip'.
			directory := mcPlatform directoryFromPath: (cachePath := archive members first fileName) relativeTo: githubCacheDirectory.
			directory exists
				ifFalse: [ MetacelloPlatform current extractRepositoryFrom: '/tmp/github.zip' to: githubCacheDirectory fullName ].
			self downloadCache at: downloadCacheKey put: cachePath.
			projectDirectory := mcPlatform directoryFromPath: cachePath relativeTo: githubCacheDirectory ].
	^ projectDirectory! !

!MCGitHubRepository class methodsFor: 'system startup' stamp: 'dkh 6/16/2012 08:24'!
startUp: resuming
    "Flush the GitHub download cache"

    resuming
        ifTrue: [ self flushDownloadCache ]! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
asRepositorySpecFor: aMetacelloMCProject
    ^ aMetacelloMCProject repositorySpec
        description: self description;
        type: 'github';
        yourself! !

!MCGitHubRepository methodsFor: 'descriptions' stamp: 'dkh 5/16/2012 14:52:41'!
description
    | desc |
    desc := self class description , self projectPath , ':' , self projectVersion.
    self repoPath ifNil: [ ^ desc ].
    ^ desc , '/' , self repoPath! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'ChristopheDemarey 4/29/2013 15:39'!
directory
    directory
        ifNil: [ 
            directory := self class projectDirectoryFrom: self projectPath version: self projectVersion.
            self repoPath ifNotNil: [ directory := MetacelloPlatform current directoryFromPath: self repoPath relativeTo: directory ] ].
    ^ directory! !

!MCGitHubRepository methodsFor: 'descriptions' stamp: 'dkh 6/16/2012 09:03'!
directoryDescription
    ^ self directory pathName! !

!MCGitHubRepository methodsFor: 'initialize-release' stamp: 'dkh 6/16/2012 09:11'!
flushCache
    "the directory acts like a cache since we download the directory from github"

    super flushCache.
    self class flushDownloadCache.
    directory := nil! !

!MCGitHubRepository methodsFor: 'initialize-release' stamp: 'dkh 7/24/2012 21:52'!
flushForScriptGet
    self class flushProjectEntry: self projectPath version: self projectVersion.
    directory := nil! !

!MCGitHubRepository methodsFor: 'comparing' stamp: 'dkh 6/16/2012 08:57'!
hash
    ^ self description hash! !

!MCGitHubRepository methodsFor: 'initialize-release' stamp: 'dkh 6/16/2012 09:09'!
initialize
    super initialize.
    directory := nil.	"we'll lazily create the directory instance"
    self readonly: true! !

!MCGitHubRepository methodsFor: 'testing' stamp: 'dkh 6/16/2012 09:00'!
isValid
    ^ true! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
metacelloProjectClassFor: aScriptEngine
    ^ MetacelloMCBaselineProject! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
projectPath
    ^ projectPath! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
projectPath: anObject
    projectPath := anObject! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
projectVersion
    (projectVersion == nil or: [ projectVersion isEmpty ])
        ifTrue: [ projectVersion := 'master' ].
    ^ projectVersion! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
projectVersion: anObject
    projectVersion := anObject! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
repoPath
    ^ repoPath! !

!MCGitHubRepository methodsFor: 'accessing' stamp: 'dkh 5/16/2012 14:52:41'!
repoPath: anObject
    repoPath := anObject! !
MCGitHubRepository initialize!
